const jwtSecret   = 'AjnjLhjxM'

const express           = require('express');
const express_graphql   = require('express-graphql');

const { buildSchema, printSchema } = require('graphql');
const expand = require('mm-graphql/expand')
const fs     = require('fs')
const uploadPath = `${__dirname}/public/images/`;
const upload  = require('multer')({ dest: uploadPath })


;(async () => {

    const {Savable, slice, getModels} = await require('./models.js')()
    const { jwtGQL, jwtCheck } = require('mm-graphql/jwt')

    const {anonSchema, anonResolvers} = require('./anon')({Savable, secret: jwtSecret})

    let schema = buildSchema(`
        type User {
             _id: String
             createdAt: String
             login: String
             nick : String
             avatar: Image
             likes: [Like]
             likesCount: Int
             incomings: [Direct]
             followers: [User]
             following: [User]
        }

        input UserInput {
             _id: String
             login: String
             nick : String
             avatar: ImageInput
             following: [UserInput]
        }

        type Like {
            _id: ID,
            post: Post,
            comment: Comment,
            direct: Direct,
            owner: User
        }

        input LikeInput {
            _id: ID,
            post: PostInput,
            comment: CommentInput,
            direct: DirectInput,
            user: UserInput,
        }

        type Post {
            _id: ID,
            createdAt: String
            title: String
            text: String,
            images: [Image],
            comments: [Comment],
            directs: [Direct],
            collections: [Collection]
            likes: [Like]
            likesCount: Int
            owner: User
        }

        input PostInput {
            _id: ID,
            title: String,
            text: String,
            images: [ImageInput],
            comments: [CommentInput],
            directs: [DirectInput],
            collections: [CollectionInput]
        }

        type Image {
            _id: ID,
            text: String,
            url: String,
            originalFileName: String,
            userAvatar: User,
            posts: [Post],
            directs: [Direct]
            owner: User
        }

        input ImageInput {
            _id: ID,
            text: String,
            userAvatar: UserInput,
            posts: [PostInput],
            directs: [DirectInput]
        }

        type Comment {
            _id: ID,
            createdAt: String,
            text: String,
            post: Post,
            answers: [Comment],
            answerTo: Comment
            likes: [Like]
            likesCount: Int
            owner: User
        }

        input CommentInput {
            _id: ID,
            text: String,
            post: PostInput,
            answers: [CommentInput],
            answerTo: CommentInput
        }

        type Direct {
            _id: ID,
            createdAt: String,
            text: String,
            post: Post,
            image: Image,
            likes: [Like]
            likesCount: Int
            to: User
            owner: User
        }

        input DirectInput {
            _id: ID,
            text: String,
            post: PostInput,
            image: ImageInput,
            likes: [LikeInput]
            to: UserInput
        }

        type Collection {
            _id: ID,
            text: String,
            posts: [Post]
            owner: User
        }

        input CollectionInput {
            _id: ID,
            text: String,
            posts: [PostInput]
        }
    `);

    schema = expand(schema)
    console.log(printSchema(schema))

    const app = express();
    app.use(express.static('public'));
    app.use('/graphql', express_graphql(jwtGQL({anonSchema, anonResolvers, schema, createContext: getModels, graphiql: true, secret: jwtSecret})))


    app.post('/upload', upload.single('photo'), async (req, res, next) => {
        let decoded;
        console.log('wtf')
        if (decoded = jwtCheck(req, jwtSecret)){
            console.log('SOME UPLOAD', decoded, req.file)

            let {models: {Image }} = await getModels(decoded.sub)
            let image = await Image.fromFileData(req.file)
            res.end(JSON.stringify({_id: image._id, url: image.url}))
        }
        else {
            res.status(503).send('permission denied')
        }
    })

    app.use(express.static('public'));


    let socketPath = "/home/asmer/node_hosts/shop"
    app.listen(socketPath, () => {
        console.log('Express GraphQL Server Now Running On localhost:4000/graphql');
        fs.chmodSync(socketPath, '777');
    });
})()

